/* This file is part of the ili2fme project.
 * For more information, please see <http://www.interlis.ch>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package ch.interlis.iom_j.itf;

import java.io.IOException;
import ch.interlis.iox.IoxException;

/**
 * @author ce
 * @version $Revision: 1.0 $ $Date: 27.07.2006 $
 */
public class ItfRawWriter {
	private java.io.Writer os=null;
	private String undefinedCode="@";
	private char continueCode='\\';
	private boolean isFormatFree=true;
	private int fixedTidSize;
	private int fixedLineSize;
	private int currentOutputColumn;
	public ItfRawWriter(java.io.OutputStream out,ch.interlis.ili2c.metamodel.Ili1Format format)
	throws java.io.UnsupportedEncodingException
	,java.io.IOException
	{
		this.os=new java.io.BufferedWriter(new java.io.OutputStreamWriter(out,ItfScanner.ITF_CHARSET));
		undefinedCode=ModelUtilities.code2string(format.undefinedCode);
		continueCode=ModelUtilities.code2string(format.continueCode).charAt(0);
		isFormatFree=format.isFree;
		if(!isFormatFree){
			fixedTidSize=format.tidSize;
			fixedLineSize=format.lineSize;
			if(fixedTidSize+5+2>fixedLineSize){
				throw new IllegalArgumentException("fixedTidSize+5+2>fixedLineSize");
			}
		}
	}
	public void close()
	throws IoxException 
	{
		flush();
		os=null;
	}
	public void flush()
		throws IoxException 
	{
		if(os!=null){
			try{
				os.flush();
			}catch(IOException ex){
				throw new IoxException("failed to flush output stream",ex);
			}
		}
	}

	/** current line seperator
	 *
	 */
	static private String nl=null;
	
	/** current line number
	 *
	 */
	private int lineNumber=0;

	/** write a line seperator
	 *
	 */
	private void newline()
	throws java.io.IOException
	{
		if(nl==null)nl=System.getProperty("line.separator");
		os.write(nl);lineNumber++;
	}
	public void writeRawText(String text)
	throws java.io.IOException
	{
		os.write(text);
		currentOutputColumn+=text.length();
	}
	public void writeNewline()
		throws java.io.IOException
	{
		if(!isFormatFree){
			os.write(ch.ehi.basics.tools.StringUtility.STRING(fixedLineSize-currentOutputColumn,' '));
		}
		newline();
		currentOutputColumn=0;
	}
	public void writeTid(String tid)
		throws java.io.IOException
	{
		writeValue(tid,true,fixedTidSize);
	}
	public void writeValue(String value,boolean isNum,int size)
		throws java.io.IOException
	{
		if(isFormatFree){
			if(value!=null){
				os.write(" "+value);
				currentOutputColumn+=1+value.length();
			}else{
				os.write(" "+undefinedCode);
				currentOutputColumn+=2;
			}
		}else{
			if(currentOutputColumn+2+size>fixedLineSize){
				os.write(ch.ehi.basics.tools.StringUtility.STRING(fixedLineSize-currentOutputColumn,' ')+continueCode);
				newline();
				os.write("CONT");
				currentOutputColumn=4;
			}
			if(value!=null){
				if(isNum){
					value=ch.ehi.basics.tools.StringUtility.STRING(size,' ')+value;
				}else{
					value=value+ch.ehi.basics.tools.StringUtility.STRING(size,' ');
				}
				value=value.substring(0,size);
				os.write("  "+value);
				currentOutputColumn+=2+size;
			}else{
				os.write(" "+undefinedCode+ch.ehi.basics.tools.StringUtility.STRING(size,' '));
				currentOutputColumn+=2+size;
			}
		}
	}
	public void writeExplanation(String explanation)
	throws IOException
	{
		if(explanation==null)return;
		explanation=explanation.trim();
		if(explanation.length()==0)return;
		java.io.LineNumberReader lines=new java.io.LineNumberReader(new java.io.StringReader(explanation));
		String line;
		while((line=lines.readLine())!=null){
			os.write(line);
			newline();
		}
	}

}
